@@ -128,7 +128,6 @@ function enigmailBuildList(refresh) {
 
   var keyListObj = {};
 
-
   EnigLoadKeyList(refresh, keyListObj, getSortColumn(), getSortDirection());
 
   gKeyList = keyListObj.keyList;
@@ -809,18 +808,31 @@ function enigmailExportKeys() {
     return;
   }
 
-  var exportFlags = 0;
-  if (gKeyList[keyList[0]].secretAvailable) {
-    var r=EnigLongAlert(EnigGetString("exportSecretKey"), null, EnigGetString("keyMan.button.exportPubKey"), EnigGetString("keyMan.button.exportSecKey"), ":cancel");
-    switch (r) {
-    case 1:
-      exportFlags |= nsIEnigmail.EXTRACT_SECRET_KEY;
-      break;
-    case 2:
-      return;
+  // check whether we want to export a private key anywhere in the key list
+  var secretFound = false;
+  for (var i=0; i<keyList.length && !secretFound; ++i) {
+    if (gKeyList[keyList[i]].secretAvailable) {
+      secretFound = true;
     }
   }
 
+  var exportFlags = 0;
+  if (secretFound) {
+    // double check that also the pivate keys shall be exportet
+    var r=EnigLongAlert(EnigGetString("exportSecretKey"), null,
+                        EnigGetString("keyMan.button.exportPubKey"),
+                        EnigGetString("keyMan.button.exportSecKey"),
+                        ":cancel");
+    switch (r) {
+      case 0: // export pub key only
+        break;
+      case 1: // export secret key
+        exportFlags |= nsIEnigmail.EXTRACT_SECRET_KEY;
+        break;
+      case 2: // cancel
+        return;
+      }
+  }
 
   var enigmailSvc = GetEnigmailSvc();
   if (!enigmailSvc)
@@ -1069,25 +1081,33 @@ function enigmailDowloadContactKeysEngine() {
     let addressBook = allAddressBooks.getNext().QueryInterface(Ci.nsIAbDirectory);
 
     if (addressBook instanceof Ci.nsIAbDirectory) { // or nsIAbItem or nsIAbCollection
+      // ask for confirmation for each address book:
+      var doIt = EnigmailCommon.confirmDlg(window,
+                   EnigGetString("downloadContactsKeys.importFrom", addressBook.dirName),
+                   EnigGetString("dlgYes"),
+                   EnigGetString("dlg.button.skip"));
+      if (!doIt) {
+        continue;  // SKIP this address book
+      }
 
       let allChildCards = addressBook.childCards;
 
       while (allChildCards.hasMoreElements()) {
 
-	      let card = allChildCards.getNext().QueryInterface(Ci.nsIAbCard);
+        let card = allChildCards.getNext().QueryInterface(Ci.nsIAbCard);
 
         try {
-	        let email = card.getPropertyAsAString("PrimaryEmail");
-	        if (email) {
-	          emails.push(email);
+          let email = card.getPropertyAsAString("PrimaryEmail");
+          if (email && email.indexOf("@")>=0) {
+            emails.push(email);
           }
         }
         catch (e) {}
 
         try {
-	        let email = card.getPropertyAsAString("SecondEmail");
-	        if (email) {
-	          emails.push(email);
+          let email = card.getPropertyAsAString("SecondEmail");
+          if (email && email.indexOf("@")>=0) {
+            emails.push(email);
           }
         }
         catch (e) {}
@@ -1096,6 +1116,25 @@ function enigmailDowloadContactKeysEngine() {
     }
   }
 
+  // list of emails might be emoty here, in which case we do nothing
+  if (emails.length <= 0) {
+    return;
+  }
+
+  // sort the e-mail array
+  emails.sort();
+
+  //remove duplicates
+  var i = 0;
+  while (i<emails.length-1) {
+    if (emails[i] == emails[i+1]) {
+      emails.splice(i,1);
+    }
+    else {
+      i = i + 1;
+    }
+  }
+
   var inputObj = {
     searchList : emails
   }
@@ -1231,7 +1270,8 @@ function determineHiddenKeys(keyObj, showInvalidKeys, showUntrustedKeys, showOth
   return show;
 }
 
-function enigApplyFilter() {
+function enigApplyFilter()
+{
   var searchTxt=gSearchInput.value;
   var nothingFoundElem = document.getElementById("nothingFound");
   nothingFoundElem.hidePopup();
@@ -1243,11 +1283,31 @@ function enigApplyFilter() {
     showOrHideAllKeys();
     return;
   }
-  else {
-    document.getElementById("emptyTree").hidePopup();
+  document.getElementById("emptyTree").hidePopup();
+
+  // skip leading 0x in case we search for a key:
+  if (searchTxt.length > 2 && searchTxt.substr(0,2).toLowerCase() == "0x") {
+    searchTxt = searchTxt.substr(2);
   }
 
   searchTxt = searchTxt.toLowerCase();
+  searchTxt = searchTxt.replace(/^(\s*)(.*)/, "$2").replace(/\s+$/,"");  // trim spaces
+
+  // check if we search for a fingerprint 
+  var fpr = null;
+  if (searchTxt.length == 49) { // possible fingerprint with spaces? 
+    if (searchTxt.search(/^[0-9a-f ]*$/)>=0 && searchTxt[4]==' ' && searchTxt[9]==' ' && searchTxt[14]==' '
+                                            && searchTxt[19]==' ' && searchTxt[24]==' ' && searchTxt[29]==' '
+                                            && searchTxt[34]==' ' && searchTxt[39]==' ' && searchTxt[44]==' ') {
+      fpr = searchTxt.replace(/ /g,"");
+    }
+  }
+  else if (searchTxt.length == 40) { // possible fingerprint without spaces
+    if (searchTxt.search(/^[0-9a-f ]*$/)>=0) {
+      fpr = searchTxt;
+    }
+  }
+
   var foundResult = false;
   var node=getFirstNode();
   while (node) {
@@ -1260,12 +1320,17 @@ function enigApplyFilter() {
           foundResult = true;
         }
     }
-    for (var subUid=0; subUid < gKeyList[node.id].SubUserIds.length; subUid++) {
+    if (hideNode==true && fpr != null && gKeyList[node.id].fpr.toLowerCase() == fpr) {
+      hideNode = false;
+      foundResult = true;
+    }
+    for (var subUid=0; hideNode==true && subUid < gKeyList[node.id].SubUserIds.length; subUid++) {
       uid = gKeyList[node.id].SubUserIds[subUid].userId;
       if (uid.toLowerCase().indexOf(searchTxt) >= 0) {
         hideNode = false;
         foundResult = true;
       }
+      // ideally we should check also the sub-key-ids
     }
     node.hidden=hideNode;
     node = node.nextSibling;
@@ -1340,27 +1405,27 @@ function getSortDirection() {
 
 function sortTree(column) {
 
-	var columnName;
-	var order = getSortDirection();
+  var columnName;
+  var order = getSortDirection();
 
-	//if the column is passed and it's already sorted by that column, reverse sort
-	if (column) {
-		columnName = column.id;
-		if (gUserList.getAttribute("sortResource") == columnName) {
-			order *= -1;
-		}
-		else {
-  		document.getElementById(gUserList.getAttribute("sortResource")).removeAttribute("sortDirection");
-  		order = 1;
-		}
-	} else {
-		columnName = gUserList.getAttribute("sortResource");
-	}
-	gUserList.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
-	gUserList.setAttribute("sortResource", columnName);
-	document.getElementById(columnName).setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
+  //if the column is passed and it's already sorted by that column, reverse sort
+  if (column) {
+    columnName = column.id;
+    if (gUserList.getAttribute("sortResource") == columnName) {
+      order *= -1;
+    }
+    else {
+      document.getElementById(gUserList.getAttribute("sortResource")).removeAttribute("sortDirection");
+      order = 1;
+    }
+  } else {
+    columnName = gUserList.getAttribute("sortResource");
+  }
+  gUserList.setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
+  gUserList.setAttribute("sortResource", columnName);
+  document.getElementById(columnName).setAttribute("sortDirection", order == 1 ? "ascending" : "descending");
   enigmailClearTree();
-	enigmailBuildList(false);
+  enigmailBuildList(false);
   enigApplyFilter();
 }
 
@@ -1371,7 +1436,7 @@ function getSortColumn() {
   case "keyCol": return "keyidshort";
   case "typeCol": return "keytype";
   case "validityCol": return "validity";
-  case "trustCol": return "trust";
+  case "trustCol": return "trust";  // ownerTrust
   case "expCol": return "expiry";
   case "fprCol": return "fpr";
   default: return "?";
